var app = app || {};
app.LocalisationService = (function (app) {
    var DEFAULT_LANGUAGE = "en";
    var instance = {};
    
    var defaultLanguageResponse = null;

    instance.getCurrentLanguage = function() {
        var webLinkSettings = WebLink.info.getSettings();
        var selectedLanguage = webLinkSettings.language;
        if (selectedLanguage === "default") {
            selectedLanguage = window.navigator.language.substr(0, 2);
        }

        return selectedLanguage;
    };

    function readFile(language, resultHandler, errorHandler) {
        var fileName = "./locales/" + language.toLowerCase() + ".json";
        var xhr = new XMLHttpRequest();
        xhr.onreadystatechange = function () {
            /*
             Check for error code 0, because on iOS when making XMLHTTPRequests to a local file,
             the received status is 0, regardless of the success of the request itself. I.e. even if the file is
             not found, the status is 0.
             */
            if(this.readyState === 4 ) {
                if (this.status === 200 || (this.status === 0 && this.responseText.length > 0)) {
                    var response = JSON.parse(xhr.responseText);
                    resultHandler(response);
                } else {
                    errorHandler({requestStatusCode: this.status, fileName: fileName})
                }
            }
        };
    
        xhr.open("GET", fileName , true);
        xhr.send();
    }

    function getDefaultMessage(key, resultHandler) {
        var message;
        if(defaultLanguageResponse != null) {
            message = defaultLanguageResponse[key];
            resultHandler(message);
        } else {
            readFile(DEFAULT_LANGUAGE, function (responce) {
                defaultLanguageResponse = responce;
                message = typeof (responce[key]) !== 'undefined' ? responce[key] : key;
                resultHandler(message);
            }, function (error) {
                console.log('Cannot load default language. ', error);
                resultHandler(key);
            })
        }
    }
    
     instance.getMessage = function(key, resultHandler) {
         var language = instance.getCurrentLanguage();
         readFile(language, function (response) {
             var message = response[key];
             if(typeof (message) !== 'undefined') {
                 resultHandler(message);
             } else {
                 getDefaultMessage(key, resultHandler);
             }
         }, function (error) {
             console.log(error);
             getDefaultMessage(key, resultHandler);                 
        });
    };

    return instance;
})(app);
