if (typeof (WebLink) === 'undefined') { WebLink = {}; }

WebLink.getDocumentProperties = function() {
    var favIcon = "";
    var link = document.querySelector('*[rel="shortcut icon"]');
    if(link) {
        favIcon = window.location.protocol + '//' + window.location.hostname + link.getAttribute('href');
    } else {
        favIcon = window.location.protocol + '//' + window.location.hostname + '/favicon.ico';
    }

    var title = document.title;

    WebLinkHostBrowser.onDocumentInfo(title, favIcon);
};

WebLink.wlScroll = (function () {
    var elements = [];
    var indexOfCurrentElement = 0;
    var currentHtmlTag = null;
    var timeOut = null;
    var border = document.createElement('div');
    var borderCSS = border.style;
    borderCSS.border = 'solid 2px red';
    borderCSS.position = 'fixed';
    borderCSS.pointerEvents = 'none';
    borderCSS.boxSizing = 'border-box';

    var onScroll = function(data) {
       var string = JSON.stringify(data);
       WebLinkHostBrowser.onScroll(string);
    };

    document.addEventListener('touchstart', function (ev) {
        var el =  getFirstScrollableElement(ev.target);
        timeOut = setTimeout(function () {
            var hasElement = false;
            if(el) {
                for(var i = 0; i < elements.length; i++) {
                    if(elements[i].htmlTag == el) {
                        hasElement = true;
                        setCurrentElement(i);
                        break;
                    }
                }

                if(!hasElement) {
                    elements.push(mapElement(el));
                    setCurrentElement(elements.length - 1);
                }
            }
        }, 600)
    });

    document.addEventListener('touchend', function (ev) {
        clearTimeout(timeOut);
        timeOut = null;
    });

    function mapElement(element) {
        var rect = element.getBoundingClientRect();
        return {
            x: rect.x,
            y: rect.y,
            width: rect.width,
            height: rect.height,
            htmlTag: element == document.documentElement ? document.scrollingElement : element
        };
    }

    function getFirstScrollableElement(element) {
        var parent = element.parentElement;

        if(isScrollable('y', element)) {
            return element;
        }

        if(parent) {
            if(isScrollable('y', parent)) {
                return parent;
            }
        } else {
            return null;
        }

        return getFirstScrollableElement(parent);
    }

    function isScrollable(direction, element) {
        var result = element.clientHeight < element.scrollHeight && Math.abs(element.clientHeight - element.scrollHeight) > 10;

        if(document.documentElement == element) {
            return result;
        }

        var overflow = window
            .getComputedStyle(element)
            .getPropertyValue('overflow-' + direction);

        result = result && (overflow === 'scroll' || overflow === 'auto');

        return result;
    }

    function getScrollableElements() {
        function getChildren(element) {
            if (element instanceof Element && isScrollable('y', element)) {
                elements.push(mapElement(element));
            }

            var children = element.children;
            for (var i = 0; i < children.length; i++) {
                getChildren(children[i]);
            }
        }

        getChildren(document.documentElement);
        elements.sort(function (el1, el2) {
            if(el1.htmlTag == document.documentElement || el1.htmlTag == document.body) {
                return -1;
            }

           return el1.x - el2.x;
        });

        elements.sort(function (el1, el2) {
            if(el1.htmlTag == document.documentElement || el1.htmlTag == document.body) {
                return -1;
            }

            return el1.y - el2.y;
        });

        if(elements.length > 0) {
            setCurrentElement(0);
        }

        window.elements = elements;
        return elements;
    }

    function setCurrentElement(index) {
        element = elements[index];
        currentHtmlTag = element.htmlTag;
        indexOfCurrentElement = index;
        /*console.log('The current htmlTag was set', currentHtmlTag);*/

        /* currentElement.style.outline = 'red solid 2px'; */
        borderCSS.width = element.width + 'px';
        borderCSS.height = element.height + 'px';
        borderCSS.top = element.y + 'px';
        borderCSS.left = element.x + 'px';
        document.body.appendChild(border);
        /*console.log(border);*/

        setTimeout(function () {
            document.body.removeChild(border);
        }, 1000);
    }

    function moveToNext() {
        indexOfCurrentElement++;
        indexOfCurrentElement = indexOfCurrentElement >= elements.length ? 0 : indexOfCurrentElement;
        setCurrentElement(indexOfCurrentElement);
    }

    function moveToPrevious() {
        indexOfCurrentElement--;
        indexOfCurrentElement = indexOfCurrentElement < 0 ? elements.length - 1 : indexOfCurrentElement;
        setCurrentElement(indexOfCurrentElement);
    }

    function scrollUp() {
        var height = 0;
        if(currentHtmlTag == document.body) {
            height = currentHtmlTag.parentElement.clientHeight;
        } else {
            height = currentHtmlTag.clientHeight;
        }

        if(onScroll && onScroll instanceof Function) {
            onScroll({
                currentPosition: currentHtmlTag.scrollTop,
                ended: false});
        }

        currentHtmlTag.scrollTop -= height;
    }

    function scrollDown() {
        var height = 0;
        if(currentHtmlTag == document.body) {
            height = currentHtmlTag.parentElement.clientHeight;
        } else {
            height = currentHtmlTag.clientHeight;
        }

        currentHtmlTag.scrollTop += height;

        if(onScroll && onScroll instanceof Function) {
            onScroll({
                currentPosition: currentHtmlTag.scrollTop,
                ended: currentHtmlTag.scrollTop + currentHtmlTag.clientHeight >= currentHtmlTag.scrollHeight});
        }
    }

    getScrollableElements();

    return {
        scrollUp: scrollUp,
        scrollDown: scrollDown,
    }
})();
