var WLDriverDistraction = WLDriverDistraction || (function () {
    var UIR_NO_RESTRICTIONS = parseInt('0x0000', 16);
    var UIR_HIDE = parseInt('0x0001', 16);
    var UIR_BLOCK_DISPLAY = parseInt('0x0002', 16);
    var UIR_NO_KEYBOARD = parseInt('0x0100', 16);
    var UIR_NO_TOUCH = parseInt('0x0200', 16);
    var UIR_NO_HW_BUTTONS = parseInt('0x0400', 16);

    var DRL_NONE = 'DRL_NONE';
    var DRL_MINOR = 'DRL_MINOR';
    var DRL_MAJOR = 'DRL_MAJOR';
    var DRL_FULL = 'DRL_FULL';

    var WIDGET_TYPES = {
        WT_ANY: 'WT_ANY',
        WT_VIDEO: 'WT_VIDEO',
        WT_BUTTON: 'WT_BUTTON',
        WT_EDIT: 'WT_EDIT'
    };

    Object.freeze(WIDGET_TYPES);

    var Restriction = (function () {
        function Restriction(jsonString) {
            var obj = JSON.parse(jsonString);
            if (obj) {
                Object.defineProperty(this, 'restrictions', {
                    value: obj.restrictions || 0,
                    writable: false,
                    enumerable: true
                });

                Object.defineProperty(this, 'blockingMessages', {
                    value: obj.blockingMessages || {},
                    writable: false,
                    enumerable: true
                });
            }
        }

        Object.defineProperties(Restriction, {
            UIR_NO_RESTRICTIONS: {
                get: function() {
                    return UIR_NO_RESTRICTIONS;
                }
            },

            UIR_HIDE: {
                get: function() {
                    return UIR_HIDE;
                }
            },

            UIR_BLOCK_DISPLAY: {
                get: function() {
                    return UIR_BLOCK_DISPLAY;
                }
            },

            UIR_NO_KEYBOARD: {
                get: function() {
                    return UIR_NO_KEYBOARD;
                }
            },

            UIR_NO_TOUCH: {
                get: function() {
                    return UIR_NO_TOUCH;
                }
            },

            UIR_NO_HW_BUTTONS: {
                get: function() {
                    return UIR_NO_HW_BUTTONS;
                }
            },
        });

        Restriction.prototype.setBlockingMessages = function (messages) {
            Object.defineProperty(this, 'blockingMessages', {
                value: Object.assign({}, messages),
                writable: false,
                enumerable: true
            });
        };

        Restriction.prototype.getBlockingMessages = function (langCode) {
            if (this.blockingMessages) {
                return this.blockingMessages[langCode];
            }
        };

        return Restriction;
    })();

    var DriverDistractionLevel = {
        get DRL_NONE() {
            return DRL_NONE;
        },
        get DRL_MINOR() {
            return DRL_MINOR;
        },
        get DRL_MAJOR() {
            return DRL_MAJOR;
        },
        get DRL_FULL() {
            return DRL_FULL;
        },
    };

    function getCurrentRestrictionLevel() {
        return WebLinkHostDD.getCurrentRestrictionLevel();
    }

    function getCurrentScreenRestriction() {
        restrictionsAsJSON = WebLinkHostDD.getCurrentScreenRestriction();
        return new Restriction(restrictionsAsJSON);
    }

    function getCurrentScreenId() {
        return WebLinkHostDD.getCurrentScreenId();
    }

    function setCurrentScreenId(id) {
        WebLinkHostDD.setCurrentScreenId(id);
    }

    function getWidgetRestrictions(HTMLElement) {
        var restrictions = null;

        var elementId = HTMLElement.id;
        var classList = HTMLElement.classList;
        var widgetType = getWidgetType(HTMLElement);

        var systemWidgetId = genarateSystemWidgetId(HTMLElement);
        restrictions = getWidgetElementRestrictions(widgetType, systemWidgetId);

        if (classList.length > 0) {
            for (var i = 0; i < classList; i++) {
                var className = classList[i];
                restrictions = getWidgetElementRestrictions(widgetType, className);
            }

            if (classList > 1) {
                restrictions = getWidgetElementRestrictions(widgetType, className.value);
            }
        }

        if (elementId) {
            restrictions = getWidgetElementRestrictions(widgetType, elementId);
        }

        return restrictions;
    }

    function getWidgetElementRestrictions(widgetType, elementId) {
        elementId = elementId || null;
        var restrictionsAsJSON = WebLinkHostDD.getWidgetRestrictions(widgetType, elementId);
        return new Restriction(restrictionsAsJSON);
    }

    function registerNotification(notificationCallback) {
        var callbackName = WebLink.callbackManager.save(this, notificationCallback, false);
        WebLinkHostDD.registerNotification(callbackName);
        return callbackName;
    }

    function unregisterNotification(callbackName) {
        WebLinkHostDD.unregisterNotification(callbackName);
    }

    function genarateSystemWidgetId(HTMLElement) {
        var parent = HTMLElement.parentElement;
        var parentName = parent.tagName.toLowerCase();
        var elementName = HTMLElement.tagName.toLowerCase();
        var index = Array.prototype.indexOf.call(parent.children, HTMLElement);

        elementId = parentName + ':' + elementName + '@' + index;
    }

    function getWidgetType(HTMLElement) {
        var tagName = HTMLElement.tagName;
        switch (tagName) {
            case 'VIDEO':
                return WIDGET_TYPES.WT_VIDEO;
            case 'BUTTON':
            case 'A':
                return WIDGET_TYPES.WT_BUTTON;
            case 'INPUT':
            case 'TEXTAREA':
                return WIDGET_TYPES.WT_EDIT;
            default:
                return WIDGET_TYPES.WT_ANY;
        }
    }

    var UIRestriction = {};

    return {
        DriverDistractionManager: {
            getCurrentRestrictionLevel: getCurrentRestrictionLevel,
            getCurrentScreenRestriction: getCurrentScreenRestriction,
            getCurrentScreenId: getCurrentScreenId,
            setCurrentScreenId: setCurrentScreenId,
            getWidgetRestrictions: getWidgetRestrictions,
            getWidgetElementRestrictions: getWidgetElementRestrictions,
            registerNotification: registerNotification,
            unregisterNotification: unregisterNotification
        },
        DriverRestrictionLevelConstants: DriverDistractionLevel,
        UIRestriction: Restriction,
        WIDGET_TYPES: WIDGET_TYPES
    }
})();
