if (typeof (WebLink) === 'undefined') {
    WebLink = {};
}
if (typeof (WebLink.video) === 'undefined') {
    WebLink.video = {};
}

/**
 *   Creates a video player either on top of the webview or below it
 *   and starts playback with the provided source.
 **/
WebLink.video.play = function (source, belowWebview) {
    if (typeof (belowWebview) === 'undefined') {
        belowWebview = false;
    }
    var res = WebLinkHostVideo.playVideo(source, belowWebview);
    return res;
};

/**
 *   Registers a listener for progress events from the player.
 **/
WebLink.video.registerProgressListener = function (progressListener) {
    WebLink.video._progressListener = progressListener;
};

/**
 *   Registers a listener for completion events from the player.
 **/
WebLink.video.registerCompletionListener = function (completionListener) {
    WebLink.video._completionListener = completionListener;
};

/**
 *   Registers a listener for playback started events from the player.
 **/
WebLink.video.registerPlaybackStartedListener = function (playbackStartedListener) {
    WebLink.video._playbackStartedListener = playbackStartedListener;
};

/**
 *   Registers a listener for playback paused events from the player.
 **/
WebLink.video.registerPlaybackPausedListener = function (playbackPausedListener) {
    WebLink.video._playbackPausedListener = playbackPausedListener;
};

/**
 *   Registers a listener for click events from the player.
 **/
WebLink.video.registerClickListener = function (clickListener) {
    WebLink.video._clickListener = clickListener;
};

/**
 *   Registers a listener for click events from the player.
 **/
WebLink.video.registerPlayerPlaybackErrorListener = function(errorListener) {
  WebLink.video._errorListener = errorListener;
};

/**
 *   Hides the player
 **/
WebLink.video.hidePlayer = function () {
    WebLinkHostVideo.hidePlayer();
};

/**
 *   Shows a hidden player. It must have been created prior to this.
 **/
WebLink.video.showPlayer = function () {
    WebLinkHostVideo.showPlayer();
};

/**
 *   Stops the playback and removes the player
 **/
WebLink.video.removePlayer = function () {
    WebLinkHostVideo.removePlayer();
};

/**
 *   Creates a video player either on top of the webview or below it
 *   at the specified position and with the specified dimensions.
 *   Passing of 0, 0, 0, 0 makes a full-screen player.
 **/
WebLink.video.createPlayer = function (x, y, width, height, belowWebview) {
    if (typeof (belowWebview) === 'undefined') {
        belowWebview = false;
    }
    WebLinkHostVideo.createPlayer(x, y, width, height, belowWebview);
};

/**
 *   Starts video playback. The player has to be created before that.
 **/
WebLink.video.startVideo = function (source) {
    WebLinkHostVideo.startVideo(source);
};

/**
 *   Pauses video playback. The player has to be created before that.
 **/
WebLink.video.pauseVideo = function () {
    WebLinkHostVideo.pauseVideo();
};

/**
 *   Resumes video playback. The player has to be created before that.
 **/
WebLink.video.resumeVideo = function () {
    WebLinkHostVideo.resumeVideo();
};

/**
 *   Changes video player's position and dimensions. The video player must have been
 *   created before that. Passing of 0, 0, 0, 0 makes a full-screen player.
 **/
WebLink.video.setPlayerBounds = function (x, y, width, height) {
    WebLinkHostVideo.setPlayerBounds(x, y, width, height);
};

/**
 *  Seeks current playback position to the specified offset.
 **/
WebLink.video.seekTo = function (millisecondsFromStart) {
    WebLinkHostVideo.seekTo(millisecondsFromStart);
};

/**
 *  Retrieves the playback length in milliseconds.
 **/
WebLink.video.getDuration = function () {
    var res = WebLinkHostVideo.getDuration();
    return res;
};


/**
 *  Returns true when video is paused
 **/
WebLink.video.isPaused = function () {
    var res = WebLinkHostVideo.isPaused();
    return res;
};


/**
 *   Called by the Java code to notify about a progress event.
 **/
WebLink.video._onProgressEvent = function (millisecondsFromStart) {
    if (typeof(WebLink.video._progressListener) === 'function') {
        WebLink.video._progressListener(millisecondsFromStart);
    }
};

/**
 *   Called by the Java code to notify about a completion event.
 **/
WebLink.video._onVideoPlayerCompletedEvent = function () {
    if (typeof(WebLink.video._completionListener) === 'function') {
        WebLink.video._completionListener();
    }
};

/**
 *   Called by the Java code to notify about a playback started event.
 **/
WebLink.video._onPlaybackStartedEvent = function () {
    if (typeof(WebLink.video._playbackStartedListener) === 'function') {
        WebLink.video._playbackStartedListener();
    }
};

/**
 *   Called by the Java code to notify about a playback started event.
 **/
WebLink.video._onPlaybackPausedEvent = function () {
    if (typeof(WebLink.video._playbackPausedListener) === 'function') {
        WebLink.video._playbackPausedListener();
    }
};

/**
 *   Called by the Java code to notify about a click event.
 **/
WebLink.video._onVideoPlayerClickEvent = function () {
    if (typeof(WebLink.video._clickListener) === 'function') {
        WebLink.video._clickListener();
    }
};

/**
 *   Called by the Java code to notify about a playback error event.
 **/
WebLink.video._onPlayerErrorEvent = function(errorCode, errorDescription) {
  if (typeof(WebLink.video._errorListener) === 'function') {
    WebLink.video._errorListener(errorCode, errorDescription);
  }
};

/**
 *   if the value sets true the video view is under the webView, otherwise is over the view.
 **/
WebLink.video.belowWebview = function (isBelow) {
    Object.defineProperty(this, '_isBelow', {
        value: isBelow,
        writable: false,
        configurable: true
    });
};



if (!WebLink.video._hasInjectedFunctionality) {
    WebLink.video._hasInjectedFunctionality = true;
    (function () {
        function changeVideoPlayerPrototype(document) {
            var proto = Object.getPrototypeOf(document.createElement('video'));
            proto.original_play = proto.play;
            proto.original_pause = proto.pause;
            proto.play = MyVideo.play;
            proto.pause = MyVideo.pause;
            proto.original_set_currentTime = proto.__lookupSetter__('currentTime');
            proto.original_get_duration = proto.__lookupGetter__('duration');
            Object.defineProperty(proto, 'paused', {
                get: function() {
                    return WebLink.video.isPaused();
                },
                set: function(value) {
                    this.original_paused = value;
                }
            });

            Object.defineProperty(proto, 'currentTime', {
                 get: function () {
                     return currentTime;
                 },
                 set: function (value) {
                     console.log(value);
                     currentTime = value;
                     WebLink.video.seekTo(htmlPlayer.currentTime * 1000);
                     this.original_set_currentTime(value);
                 }
            });

            Object.defineProperty(proto, 'src', {
                 get: function () {
                     return playerSrc;
                 },
                 set: function (value) {
                     isSourceChanged = true;
                     playerSrc = value;
                 }
            });

            Object.defineProperty(proto, 'duration', {
                 get: function () {
                     var duration = WebLink.video.getDuration();
                     if(isNaN(duration)) {
                        duration = this.original_get_duration;
                     } else {
                        duration /= 1000;
                     }
                     return duration;
                 },
                 set: function (value) {}
            });
        }

        WebLink.DEBUG = false;
        /*enable overly verbose js logging */
        var htmlPlayer = null;
        var currentTime = 0;
        var playerSrc = null;
        var isSourceChanged = false;

        MyVideo = {};

        MyVideo.play = function () {
            console.log('MyVideo.play');
            if(htmlPlayer !== this) {
                currentTime = 0;
                htmlPlayer = this;
            }

            htmlPlayer = this;
            this.id = new Date().getTime();

            if(isSourceChanged) {
                isSourceChanged = false;
                WebLink.video.removePlayer();
                WebLink.video.play(playerSrc, WebLink.video._isBelow);
                console.log('WebLink.video.play');
            } else {
               if(WebLink.video.isPaused()) {
                   WebLink.video.resumeVideo();
                   console.log('WebLink.video.resumeVideo');
                   var playingEvent = new Event('playing');
                   htmlPlayer.dispatchEvent(playingEvent);
               }
            }

            this.muted = true;
            this.original_pause();
        };

        MyVideo.pause = function () {
            WebLink.video.pauseVideo();
            this.original_pause();
        };

        WebLink.video.registerProgressListener(function (milliseconds) {
            if (htmlPlayer) {
                currentTime = milliseconds / 1000;
                var timeUpdateEvent = new Event('timeupdate');
                htmlPlayer.dispatchEvent(timeUpdateEvent);
            }
        });

        WebLink.video.registerPlaybackStartedListener(function() {
            if (htmlPlayer) {
                var playingEvent = new Event('playing');
                htmlPlayer.dispatchEvent(playingEvent);
            }
        });

        WebLink.video.registerCompletionListener(function() {
            if (htmlPlayer) {
                var endedEvent = new Event('ended');
                htmlPlayer.dispatchEvent(endedEvent);
            }
        });

        WebLink.video.registerPlaybackPausedListener(function() {
             var pauseEvent = new Event('pause');
             htmlPlayer.dispatchEvent(pauseEvent);
        });

        WebLink.video.registerPlayerPlaybackErrorListener(function(code, description) {
             isSourceChanged = true;
             var errorEvent = new Event('error');
             errorEvent.error = {code: code, description: description};
             htmlPlayer.dispatchEvent(errorEvent);
        });

        changeVideoPlayerPrototype(document);
        var iFrameObserver = new MutationObserver(function (mutations) {
            mutations.forEach(function (mutation) {
                try {
                    if (mutation.type == 'childList') {
                        if (mutation.addedNodes.length >= 1) {
                            if (mutation.addedNodes[0].nodeName !== "#text") {
                                if (WebLink.DEBUG) {
                                    console.log("DOM Changed, element added")
                                }

                                if (mutation.addedNodes[0].nodeName === "IFRAME") {
                                    var ifr = mutation.addedNodes[0];
                                    var ifDoc = mutation.addedNodes[0].contentDocument || mutation.addedNodes[0].contentWindow.document;
                                    var iWindow = ifr.contentWindow;
                                    console.timeEnd('iframe');

                                    ifr.addEventListener('load', function() { this.style.visibility = 'visible'; });

                                    if (iWindow.MediaSource && iWindow.MediaSource.isTypeSupported) {
                                        iWindow.MediaSource.isTypeSupported = function () {
                                            return false;
                                        };
                                    }

                                    iWindow.MediaSource;
                                    if (!iWindow.navigator.isChanged) {
                                        iWindow.navigator.isChanged = true;
                                        Object.defineProperty(iWindow.navigator, 'userAgent', {
                                            get: function () {
                                                return 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.113 Safari/537.36';
                                            }
                                        });
                                    }

                                    if (typeof Object.getPrototypeOf(ifDoc).createElement_ori === 'undefined') {
                                        Object.getPrototypeOf(ifDoc).createElement_ori = Object.getPrototypeOf(document).createElement;
                                        if (typeof(Object.getPrototypeOf(ifDoc.createElement('video')).original_play) === 'undefined') {
                                            changeVideoPlayerPrototype(ifDoc);
                                        }
                                    }
                                }
                            }
                        } else if (mutation.removedNodes.length >= 1) {
                            if (WebLink.DEBUG) {
                                console.log("DOM changed, element removed")
                            }

                            if (mutation.removedNodes[0].nodeName === "IFRAME") {
                                WebLink.video.removePlayer();
                            }
                        }
                    } else if (mutation.type == 'attributes') {
                        if (WebLink.DEBUG) {
                            console.log("DOM attributes changed!")
                        }
                        ;
                    }
                }
                catch (err) {
                    if (WebLink.DEBUG) {
                        console.log(err.message);
                    }
                }
            });
        });

        var iFrameObserverConfig = {
            attributes: true,
            childList: true,
            characterData: true,
            subtree: true
        };

        iFrameObserver.observe(document.body, iFrameObserverConfig);
    })();
}

