if (typeof (WebLink) === 'undefined') {
    WebLink = {};
}

WebLink.inputTypes = ['email', 'number', 'password', 'search', 'tel', 'text', 'url'];
WebLink.keyboardInput = WebLink.keyboardInput || null;
WebLink.lastTouchDiv = WebLink.lastTouchDiv || null;

if (typeof (WebLink.appManager) === 'undefined') {
    WebLink.appManager = {};
}

(function (WebLink) {
     /*------------ Keyboard support*/
     if (typeof (WebLink.checkForKeyboardInput) === 'undefined') {
         var SHOW_KEYBOARD_ATTRIBUTE = 'show-keyboard-for';
         var DELAY_KEYBOARD_ATTRIBUTE = 'show-keyboard-delay';
         var observer = null;
         var lastKeyboardTimeout;

         function getParentWithAttributeShowKeyboardFor(element) {
             var attributeValue = element.getAttribute(SHOW_KEYBOARD_ATTRIBUTE);
             if (attributeValue) {
                 return element;
             }

             var parent = element.parentElement;
             if (parent == null) {
                 return null;
             }

             return getParentWithAttributeShowKeyboardFor(parent);
         }

         function startObserver(elementId, callback) {
             if (observer) {
                 observer.disconnect();
             }

             observer = new MutationObserver(function (mutations, observer) {
                 var element;
                 for (var mutation of mutations) {
                     var i  = 0;
                     for (i = 0; i < mutation.addedNodes.length; i++) {
                         if (mutation.addedNodes[i].querySelector) {
                             element = mutation.addedNodes[i].querySelector('*[id='+ elementId +']');
                             if (element) {
                                 callback(element);
                                 observer.disconnect();
                                 return;
                             }
                         }
                     }

                     for (i = 0; i < mutation.removedNodes.length; i++) {
                         if (mutation.removedNodes[i].querySelector) {
                             element = mutation.removedNodes[i].querySelector('*[id='+ elementId +']');
                             if (element) {
                                 callback(null);
                                 observer.disconnect();
                                 return;
                             }
                         }
                     }
                 }
             });

             observer.observe(document.body, {childList: true, subtree: true});
         }

         WebLink.checkForKeyboardInput = function (clientX, clientY) {
             clearTimeout(lastKeyboardTimeout);
             var element = document.elementFromPoint(clientX, clientY);

             var elementWithAttributeShowKeyboardFor = getParentWithAttributeShowKeyboardFor(element);

             if (elementWithAttributeShowKeyboardFor) {
                 var inputId = elementWithAttributeShowKeyboardFor.getAttribute(SHOW_KEYBOARD_ATTRIBUTE);
                 var delay = Number(elementWithAttributeShowKeyboardFor.getAttribute(DELAY_KEYBOARD_ATTRIBUTE));

                 delay = isNaN(delay) ? 0 : delay;

                 if (inputId) {
                     element = document.getElementById(inputId);
                 }

                 startObserver(inputId, function (element) {
                     if(element != null) {
                         lastKeyboardTimeout = setTimeout(function() {
                              WebLink.checkElementForKeyboardInput(element);
                         }, delay)
                     } else {
                         WebLinkHost.hideKeyboard();
                     }
                 });
             }

             WebLink.checkElementForKeyboardInput(element);
         };
     }
    /*------------ Keyboard support*/
})(WebLink);

WebLink.checkElementForKeyboardInput = function (element) {
    if (element != null) {
        console.log(element.nodeName.toLowerCase());
    }
    if (element != null && ((element.nodeName.toLowerCase() === 'input' && this.inputTypes.indexOf(element.type) != -1) || element.nodeName.toLowerCase() === 'textarea')) {
        WebLink.keyboardInput = element;
        element.readOnly = true;
        WebLinkHost.showKeyboard(element.type, WebLink.GetBoundingClientRectZoomed(element).bottom, WebLink.keyboardInput.value);
        WebLink.keyboardInput.focus();
    }
    else {
        WebLinkHost.hideKeyboard();
        /*WebLink.keyboardInput = null;*/
    }
};

WebLink.setKeyboardText = function (text) {
    if (WebLink.keyboardInput) {
        WebLink.keyboardInput.value = text;
        WebLink.keyboardInput.blur();
        WebLink.keyboardInput.focus();

        /*
         *  A fix for Angular;
         *  also other code may depend on notifications for changed value of an input
         */
        var evt = document.createEvent("HTMLEvents");
        evt.initEvent("input", false, true);
        WebLink.keyboardInput.dispatchEvent(evt);
    }
};


WebLink.jsTouchAndKeyboard = true;
WebLink.eventTypes = WebLink.eventTypes || {
    'touch': ['touchstart', 'touchmove', 'touchend'],
    'mouse': ['mousedown', 'mousemove', 'mouseup', 'mouseover', 'mouseout', 'click']
};
WebLink.showKeyboard = false;
WebLink.hideKeyboard = false;
WebLink.RecursiveFindPosition = WebLink.RecursiveFindPosition || function (element) {
    if (element) {
        var total = WebLink.RecursiveFindPosition(element.offsetParent);
        var zoom = window.getComputedStyle(element)["zoom"];
        if (!zoom) zoom = 1.0;
        total.zoom *= zoom;
        total.left += total.zoom * (element.offsetLeft - element.scrollLeft + element.clientLeft);
        total.top += total.zoom * (element.offsetTop - element.scrollTop + element.clientTop);
        return total;
    }
    return {left: 0, top: 0, zoom: 1.0};
};
WebLink.GetBoundingClientRectZoomed = WebLink.GetBoundingClientRectZoomed || function (element) {
    var baseRect = element.getBoundingClientRect();
    var result = WebLink.RecursiveFindPosition(element);
    result.width = baseRect.width * result.zoom;
    result.height = baseRect.height * result.zoom;
    result.right = result.left + result.width;
    result.bottom = result.top + result.height;
    return result;
};
WebLink.GetSearchRect = WebLink.GetSearchRect || function (direction, elementRect) {
    var center = WebLink.CenterPointForRect(elementRect);
    return {
        left: ((direction === 'right') ? center.x : 0),
        right: ((direction === 'left') ? center.x : 800),
        top: ((direction === 'down') ? center.y : 0),
        bottom: ((direction === 'up') ? center.y : 480)
    };
};
WebLink.FindButton = WebLink.FindButton || function (direction, element) {
    if (direction === null) {
        var searchRect = WebLink.GetBoundingClientRectZoomed(element);
        var target = WebLink.RecursiveElementFind(direction, element, searchRect, document);
        return target;
    } else if (element && element.getBoundingClientRect) {
        var rect = WebLink.GetBoundingClientRectZoomed(element);
        var searchRect = WebLink.GetSearchRect(direction, rect);
        var target = WebLink.RecursiveElementFind(direction, element, searchRect, document);
        return target;
    } else {
        var searchRect = {top: 0, left: 0, bottom: 480, right: 800};
        var target = WebLink.RecursiveElementFind(direction, element, searchRect, document);
        return target;
    }
};
WebLink.CenterPointForElement = WebLink.CenterPointForElement || function (element) {
    if (element && element.getBoundingClientRect) {
        var rect = WebLink.GetBoundingClientRectZoomed(element);
        return {
            x: rect.left + (rect.right - rect.left) / 2,
            y: rect.top + (rect.bottom - rect.top) / 2
        };
    }
    return {x: 0, y: 0};
};
WebLink.CenterPointForRect = WebLink.CenterPointForRect || function (rect) {
    return {
        x: rect.left + (rect.right - rect.left) / 2,
        y: rect.top + (rect.bottom - rect.top) / 2
    };
};
WebLink.ToPointForElement = WebLink.ToPointForElement || function (direction, element) {
    if (element) {
        var rect = WebLink.GetBoundingClientRectZoomed(element);
        if (direction === 'right') {
            return {
                x: rect.left,
                y: rect.top + (rect.bottom - rect.top) / 2
            };
        } else if (direction === 'left') {
            return {
                x: rect.right,
                y: rect.top + (rect.bottom - rect.top) / 2
            };
        } else if (direction === 'up') {
            return {
                x: rect.left + (rect.right - rect.left) / 2,
                y: rect.bottom
            };
        } else if (direction === 'down') {
            return {
                x: rect.left + (rect.right - rect.left) / 2,
                y: rect.top
            };
        }
        return {
            x: rect.left + (rect.right - rect.left) / 2,
            y: rect.top + (rect.bottom - rect.top) / 2
        };
    }
    return {x: 0, y: 0};
};
WebLink.GetXBias = WebLink.GetXBias || function (direction) {
    if (direction === 'down' || direction === 'up')
        return 5.0;
    return 1.0;
};
WebLink.GetYBias = WebLink.GetYBias || function (direction) {
    if (direction === 'right' || direction === 'left')
        return 5.0;
    return 1.0;
};
WebLink.DistanceBiased = WebLink.DistanceBiased || function (direction, point1, point2) {
    var xs = 0;
    var ys = 0;
    xs = point2.x - point1.x;
    xs = xs * xs;
    ys = point2.y - point1.y;
    ys = ys * ys;
    return Math.sqrt(xs * WebLink.GetXBias(direction) + ys * WebLink.GetYBias(direction));
};
WebLink.IsPointCloser = WebLink.IsPointCloser || function (direction, original, best, next) {
    var dist_best = WebLink.DistanceBiased(direction, original, best);
    var dist_next = WebLink.DistanceBiased(direction, original, next);
    return dist_best >= dist_next;
};
WebLink.RectContainsPoint = WebLink.RectContainsPoint || function (rect, point) {
    return rect.left < point.x && rect.right > point.x && rect.top < point.y && rect.bottom > point.y;
};
WebLink.HasInteractiveEvents = WebLink.HasInteractiveEvents || function (object) {
    if (object) {
        return object['touchstart'] || object['touchend'] || object['click'] || object['mousedown'] || object['mouseup'];
    }
    return false;
};
WebLink.ShouldSelectElement = WebLink.ShouldSelectElement || function (element) {
    if (element && element.getBoundingClientRect) {
        var events = (window.jQuery) ? $._data($(element)[0], 'events') : null;
        var should = (element.href || events || element.ontouchevent || element.ontouchstart || element.onmousedown || element.onclick || element.onkeydown);
        var should = should || WebLink.HasInteractiveEvents(element.eventListenerList);
        if (element.nodeName.toLowerCase().indexOf('input') !== -1 || element.nodeName.toLowerCase().indexOf('textarea') !== -1 || element.nodeName.toLowerCase().indexOf('button') !== -1) {
            should = true;
        }
        if (should) {
            var style = window.getComputedStyle(element);
            if (style.visibility === 'hidden' || style.display === 'none') {
                should = false;
            }
        }
        return should;
    }
    return false;
};
WebLink.RecursiveElementFind = WebLink.RecursiveElementFind || function (direction, originalElement, searchRect, currentSearchElement) {
    var childBestMatchElement = null;
    if (currentSearchElement && currentSearchElement.childNodes && currentSearchElement.childNodes.length) {
        var len = currentSearchElement.childNodes.length;
        for (var i = 0; i < len; i++) {
            var child = currentSearchElement.childNodes[i];
            var childBestElement = WebLink.RecursiveElementFind(direction, originalElement, searchRect, child);
            if (childBestElement) {
                var goalPt = WebLink.CenterPointForElement(originalElement);
                var currPt = WebLink.CenterPointForElement(childBestElement);
                var bestPt = WebLink.CenterPointForElement(childBestMatchElement);
                if (!childBestMatchElement || WebLink.IsPointCloser(direction, goalPt, bestPt, currPt)) {
                    childBestMatchElement = childBestElement;
                }
            }
        }
    }
    if (childBestMatchElement) {
        return childBestMatchElement;
    }
    if (WebLink.ShouldSelectElement(currentSearchElement)) {
        var rect = WebLink.GetBoundingClientRectZoomed(currentSearchElement);
        var currPt = WebLink.CenterPointForElement(currentSearchElement);
        var area = rect.width * rect.height;
        if (area > 0 && WebLink.RectContainsPoint(searchRect, currPt)) {
            return currentSearchElement;
        }
    }
    return null;
};
WebLink.activeTouch = WebLink.activeTouch || null;
WebLink.activeElement = WebLink.activeElement || null;
WebLink.activeElementPoint = WebLink.activeElementPoint || null;
WebLink.directionFromKey = WebLink.directionFromKey || function (keyCode) {
    switch (keyCode) {
        case 18:
            return 'left';
        case 19:
            return 'up';
        case 20:
            return 'right';
        case 21:
            return 'down';
    }
    ;
    return null;
};
WebLink.keyCodeIsNavigation = WebLink.keyCodeIsNavigation || function (keyCode) {
    return keyCode === 4 || keyCode === 18 || keyCode === 19 || keyCode === 20 || keyCode === 21;
};
WebLink.elementFromPointFix = WebLink.elementFromPointFix || function (x, y) {
    var element = document.elementFromPoint(x, y);
    if (element instanceof HTMLIFrameElement) {
        var rect = WebLink.GetBoundingClientRectZoomed(element);
        element = element.contentWindow.document.elementFromPoint(x - rect.left, y - rect.top);
    }
    return element;
};
WebLink.arrowKeyNavigation = WebLink.arrowKeyNavigation || function (eventType, keyCode) {
    switch (keyCode) {
        case 4:/*enter*/
            WebLink.activeElementPoint = WebLink.CenterPointForElement(WebLink.activeElement);
            var element = WebLink.elementFromPointFix(WebLink.activeElementPoint.x, WebLink.activeElementPoint.y);
            var x = WebLink.activeElementPoint.x;
            var y = WebLink.activeElementPoint.y;
            element.focus();
            if (eventType === 'keydown') {
                WebLink.onEvent('touchstart', x, y, x, y, x, y);
            } else if (eventType === 'keyup') {
                WebLink.UnselectElement(WebLink.activeElement);
                WebLink.onEvent('touchend', x, y, x, y, x, y);
                WebLink.onEvent('mousedown', x, y, x, y, x, y);
                WebLink.onEvent('mouseup', x, y, x, y, x, y);
                if (WebLink.activeElement.eventListenerList && WebLink.activeElement.eventListenerList['tap']) {
                    var maxi = WebLink.activeElement.eventListenerList['tap'].length;
                    for (var i = 0; i < maxi; i++) {
                        var func = WebLink.activeElement.eventListenerList['tap'][i];
                        WebLink.activeElement.addEventListener('click', func);
                    }
                    WebLink.onEvent('click', x, y, x, y, x, y);
                    for (var i = 0; i < maxi; i++) {
                        var func = WebLink.activeElement.eventListenerList['tap'][i];
                        WebLink.activeElement.removeEventListener('click', func);
                    }
                } else {
                    WebLink.onEvent('click', x, y, x, y, x, y);
                }
                setTimeout(function () {
                    var element = null;
                    if ((element = WebLink.FindButton(null, WebLink.activeElement))) {
                    } else if ((element = document.getElementById(WebLink.activeElement.id))) {
                    } else if ((element = document.elementFromPoint(WebLink.activeElementPoint.x, WebLink.activeElementPoint.y))) {
                    } else {
                    }
                    WebLink.activeElement = element;
                    WebLink.SelectElement(WebLink.activeElement);
                }, 250);
            } else if (eventType === 'keypress') {
                WebLink.showKeyboard = false;
                WebLink.checkForKeyboardInput(x, y);
                if (WebLink.showKeyboard) {
                    WebLink.hideKeyboard = false;
                } else {
                    WebLink.hideKeyboard = true;
                }
            }
            break;
        case 18: /*left*/
        case 19: /*up*/
        case 20: /*right*/
        case 21: /*down*/
            var dir = WebLink.directionFromKey(keyCode);
            if (dir && eventType === 'keypress') {
                var res = WebLink.FindButton(dir, WebLink.activeElement);
                if (res) {
                    WebLink.UnselectElement(WebLink.activeElement);
                    WebLink.activeElement = res;
                    WebLink.SelectElement(WebLink.activeElement);
                }
            }
            break;
    }
    ;
};
WebLink.UnselectElement = WebLink.UnselectElement || function (element) {
    if (element) {
        var active = WebLink.GetSelectableElement(element);
        if (active) {
            active.style.setProperty('outline', '0px', 'important');
            active.style.outlineOffset = '0px';
        }
    }
};
WebLink.SelectElement = WebLink.SelectElement || function (element) {
    if (element) {
        var active = WebLink.GetSelectableElement(element);
        if (active) {
            active.style.setProperty('outline', 'solid red 5px', 'important');
            active.style.outlineOffset = '-5px';
        }
    }
};
WebLink.GetSelectableElement = WebLink.GetSelectableElement || function (element) {
    if (element.clientWidth && element.clientHeight) {
        return element;
    }
    var maxi = element.children.length;
    for (var i = 0; i < maxi; i++) {
        var child = element.children[i];
        if (child.clientWidth && child.clientHeight) {
            return child;
        }
    }
    var res = element;
    while (res) {
        res = res.parentElement;
        if (res && res.clientWidth && res.clientHeight) {
            return res;
        }
    }
    return element;
};
WebLink.keyboardEvent = WebLink.keyboardEvent || function (type, keyCode) {
    var event = null;
    switch (keyCode) {
        case 13: /* return */
            var keyboardEventInit = {code: "Enter", key: "Enter", keyCode: keyCode, bubbles: true, cancelable: true};
            event = new KeyboardEvent(type, keyboardEventInit);
            delete event.keyCode;
            Object.defineProperty(event, "keyCode", {"value": keyCode});
            WebLinkHost.hideKeyboard();
            WebLink.keyboardInput.blur();
            break;
    }
    if (event) {
        WebLink.keyboardInput.dispatchEvent(event);
    }
    WebLink.arrowKeyNavigation(type, keyCode);
};
WebLink.getEventTypeByName = WebLink.getEventTypeByName || function (name) {
    var type = null;
    for (var key in WebLink.eventTypes) {
        if (WebLink.eventTypes[key].indexOf(name) != -1) {
            type = key;
            break;
        }
    }
    return type;
};

WebLink.createTouch = WebLink.createTouch || function (identifier, target, pageX, pageY, screenX, screenY) {
    var touch = {};
    touch.identifier = identifier;
    touch.screenX = screenX;
    touch.screenY = screenY;
    touch.clientX = screenX;
    touch.clientY = screenY;
    touch.pageX = pageX;
    touch.pageY = pageY;
    touch.target = target;
    touch.radiusX = 0;
    touch.radiusY = 0;
    touch.force = 1;
    rotationAngle = 0;
    return touch;
};
WebLink.createTouchList = WebLink.createTouchList || function (touches) {
    var touchList = [];
    if (touches !== undefined && Object.prototype.toString.call(touches) === '[object Array]') {
        for (var i = 0; i < touches.length; i++) {
            touchList.push(touches[i]);
        }
    }
    touchList.item = function (index) {
        return this[index];
    };
    touchList.identifiedTouch = function (id) {
        for (var i = 0; i < this.length; i++) {
            if (this[i].identifier == id) {
                return this[i];
            }
        }
    };
    return touchList;
};

WebLink.createTouchEvent = WebLink.createTouchEvent || function (name, pageX, pageY, clientX, clientY, screenX, screenY) {
    var event = document.createEvent('Event');
    event.initEvent(name, true, true);
    event.altKey = false;
    event.ctrlKey = false;
    event.shiftKey = false;
    event.metaKey = false;
    event.charCode = 0;
    event.keyCode = 0;
    event.layerX = clientX;
    event.layerY = clientY;
    event.pageX = pageX;
    event.pageY = pageY;
    event.screenX = screenX;
    event.screenY = screenY;
    event.which = 0;
    event.rotation = 0;
    event.scale = 1.0;
    event.view = window;

    var touch = WebLink.createTouch(0, event.target, pageX, pageY, screenX, screenY);
    var touchList = WebLink.createTouchList([touch]);
    if (name == 'touchend') {
        event.touches = WebLink.createTouchList();
        event.changedTouches = touchList;
        event.targetTouches = WebLink.createTouchList();
    } else {
        event.touches = touchList;
        event.changedTouches = touchList;
        event.targetTouches = touchList;
    }
    return event;
};

/* used only for Android 4.4 */
WebLink.activateScroll = WebLink.activateScroll || function activateScroll() {
    var lastLeft = 0,
        lastTop = 0;

    function getScrollableYParent(element) {
        var parent = element.parentElement;
        if (!parent || isScrollable('y', element)) {
            return element;
        }
        if (isScrollable('y', element) && (parent.scrollHeight - parent.scrollTop >= parent.clientHeight)) {
            return parent;
        }
        return getScrollableYParent(parent);
    }

    function getScrollableXParent(element) {
        var parent = element.parentElement;
        if (!parent || isScrollable('x', element)) {
            return element;
        }
        if (isScrollable('x', parent) && (parent.scrollWidth - parent.scrollLeft >= parent.clientWidth)) {
            return parent;
        }
        return getScrollableXParent(parent);
    }

    function isScrollable(direction, element) {
        var overflow = window
            .getComputedStyle(element)
            .getPropertyValue('overflow-' + direction);
        return overflow === 'scroll' || overflow === 'auto';
    }

    function scrollElement(e) {
        var top = e.pageY;
        var left = e.pageX;
        if (!lastTop) {
            lastTop = top;
        }
        if (!lastLeft) {
            lastLeft = left;
        }
        var scrollableXElement = getScrollableXParent(e.target);
        var scrollableYElement = getScrollableYParent(e.target);
        if (scrollableYElement) {
            var deltaTop = (lastTop - top);
            lastTop = top;
            scrollableYElement.scrollTop += deltaTop;
        }
        if (scrollableXElement) {
            var deltaLeft = (lastLeft - left);
            lastLeft = left;
            scrollableXElement.scrollLeft += deltaLeft;
        }
    }

    document.addEventListener('touchstart', function (e) {
        lastLeft = 0;
        lastTop = 0;
        document.addEventListener('touchmove', scrollElement);
    });
    document.addEventListener('touchend', function (e) {
        document.removeEventListener('touchmove', scrollElement);
    })
};
WebLink.createMouseEvent = WebLink.createMouseEvent || function (name, pageX, pageY, clientX, clientY, screenX, screenY) {
    var event = document.createEvent('MouseEvents');
    event.initMouseEvent(name, true, name != 'mousemove', window, 0, screenX, screenY,
        clientX, clientY, false, false, false, false, 0, null);
    return event;
};
WebLink.onEvent = WebLink.onEvent || function (name, pageX, pageY, clientX, clientY, screenX, screenY) {
    var type = WebLink.getEventTypeByName(name);
    var event = null;
    if (type == 'touch') {
        if (!WebLink.scrollActivated) {
            WebLink.activateScroll();
            WebLink.scrollActivated = true;
        }

        event = WebLink.createTouchEvent(name, pageX, pageY, clientX, clientY, screenX, screenY);
    } else if (type == 'mouse') {
        event = WebLink.createMouseEvent(name, pageX, pageY, clientX, clientY, screenX, screenY);
    }
    var element = document.elementFromPoint(clientX, clientY);
    if (element instanceof HTMLIFrameElement) {
        var rect = WebLink.GetBoundingClientRectZoomed(element);
        element = element.contentWindow.document.elementFromPoint(clientX - rect.left,
            clientY - rect.top);
    }
    if (element != null) {
        if (name == 'touchstart') {
            WebLink.lastTouchDiv = element;
        }
        if (name == 'touchmove') {
            if (WebLink.lastTouchDiv) {
                WebLink.lastTouchDiv.dispatchEvent(event);
            } else {
                WebLink.lastTouchDiv = element;
                WebLink.lastTouchDiv.dispatchEvent(event);
                element.addEventListener('touchend', function (e) {
                    WebLink.lastTouchDiv = null;
                    e.target.removeEventListener(e.type, arguments.callee);
                })
            }
        } else {
            element.dispatchEvent(event);
        }
        if (name == 'touchstart') {
            WebLink.activateElement(element);
            WebLink.lastTouchStartEvent = event;
        }
        if (name == 'click') {
            WebLink.checkElementForKeyboardInput(element);
        }
    }
    if (name == 'touchend') {
        WebLink.deactivateElement();
        var interval = event.timeStamp - WebLink.lastTouchStartEvent.timeStamp;
        if (interval < 400) {
            var clickEvent = document.createEvent('Event');
            clickEvent.initEvent('click', true, true);
            WebLink.lastTouchDiv.dispatchEvent(clickEvent);

            var mousedownEvent = document.createEvent('MouseEvents');
            mousedownEvent.initMouseEvent("mousedown", true, true, window, 0, pageX, pageY, clientX, clientY, false, false, false, false, 0, null);
            WebLink.lastTouchDiv.dispatchEvent(mousedownEvent);

            var mouseupEvent = document.createEvent('MouseEvents');
            mouseupEvent.initMouseEvent("mouseup", true, true, window, 0, pageX, pageY, clientX, clientY, false, false, false, false, 0, null);
            WebLink.lastTouchDiv.dispatchEvent(mouseupEvent);
        }
    }
    if (WebLink.showKeyboard == true) {
        WebLink.showKeyboard = false;
        return 'showKeyboard';
    }
};
WebLink.activateElement = WebLink.activateElement || function (element) {
    element.focus();
    var maxBubble = 3;
    var bubble = 0;
    while (bubble < maxBubble && element.parentElement != null) {
        element.className += ' WebLinkActive';
        element = element.parentElement;
        bubble += 1;
    }
};
WebLink.deactivateElement = WebLink.deactivateElement || function () {
    var elements = document.getElementsByClassName('WebLinkActive');
    for (var i = elements.length - 1; i >= 0; --i) {
        var element = elements[i];
        element.className = element.className.replace(/ WebLinkActive/g, '');
    }
};


/*------------ Select elements support*/
WebLink.dateInputTypes = ['date', 'time', 'week', 'datetime', 'datetime-local'];
WebLink.onSelectTouchStartFunc = function (e) {
    console.log("onSelectTouchStartFunc");
    e.preventDefault();
};
WebLink.onSelectTouchEndFunc = function (e) {
    console.log("onSelectTouchEndFunc");
    e.preventDefault();
};
WebLink.onSelectClickFunc = function (e) {
    console.log("onSelectClickFunc");
    e.preventDefault();
};
WebLink.checkSelectElement = function (element) {
    if (element != null && element.nodeName.toLowerCase() === 'select') {
        element.addEventListener('touchstart', WebLink.onSelectTouchStartFunc, false);
        element.addEventListener('touchend', WebLink.onSelectTouchEndFunc, false);
        element.addEventListener('click', WebLink.onSelectClickFunc, false);
        console.log(".............. SELECT element processed!");
    }
    if (element != null && element.nodeName.toLowerCase() === 'input' && this.dateInputTypes.indexOf(element.type) != -1) {
        console.log('Making read-only: ' + element.nodeName.toLowerCase());
        element.readOnly = true;
    }
};
WebLink.focusHappened = function () {
    /*WebLink.checkElementForKeyboardInput(document.activeElement);*/
};
if (window.onFocusListenerAdded === undefined) {
    window.onFocusListenerAdded = true;
    if (typeof(window.addEventListener) === 'function') {
        window.addEventListener('focus', WebLink.focusHappened, true);
    } else {
        window.attachEvent('onfocusin', WebLink.focusHappened);
    }
}

var all = document.getElementsByTagName("*");
for (var i = 0, max = all.length; i < max; i++) {
    var element = all[i];
    WebLink.checkSelectElement(element);
}

WebLink.checkChildrenElement = function (elem) {
    var childNodes = elem.childNodes;
    for (var index = 0; index < childNodes.length; index++) {
        WebLink.checkChildrenElement(childNodes[index]);
    }
};

document.addEventListener('DOMNodeInserted', function (event) {
    var element = event.target;
    WebLink.checkSelectElement(element);
    WebLink.checkChildrenElement(element);
});


WebLink.getLocalApplications = function (callback) {
    return WebLinkHost.getLocalApplications(callback);
};

window.close = function () {
    window.document.location.href = "wlclose://";
};

WebLink.registerForVoiceCommands = WebLink.registerForVoiceCommands || function (grammarUri, callback) {
    WebLinkHost.registerForVoiceCommands(grammarUri, callback);
};

window.focus = function () {
    WebLinkHost.bringWindowToFocus();
};

WebLink.getApplications = function (callback) {
    return WebLinkHost.getWLApplications(callback);
};

setTimeout(function () {
    if (window.jQuery) {
        $('input').attr('readonly', 'readonly');
        $('textarea').attr('readonly', 'readonly');
    }
}, 1500);

WebLink.callbackManager = WebLink.callbackManager || (function (undefined) {
    var instance = {};
    instance.save = function (scope, obj, shouldEraseAfterFirstCall) {
        var identifier = 'callback';
        while (instance[identifier]) {
            identifier += Math.floor(Math.random() * 10);
        }
        var shouldErase = true;
        if (typeof(shouldEraseAfterFirstCall) !== 'undefined') {
            shouldErase = shouldEraseAfterFirstCall;
        }
        instance[identifier] = {fn: obj, scope: scope, shouldErase: shouldErase};
        console.error("Save id " + identifier + " from storage.");
        return identifier;
    };

    instance.callFunction = function (identifier, optFn, args) {
        var entry = instance[identifier];
        args = args || [];

        if (entry) {
            if (typeof (entry.fn) === "function") {
                entry.fn.apply(entry.scope || window, args);
                if (entry.shouldErase) {
                    instance.remove(identifier);
                }
            } else if (typeof (entry.fn) === "object" && typeof(entry.fn.nodeName) !== "undefined" && entry.fn.nodeName.toLowerCase() === "audio" && optFn === "play") {
                entry.fn.play();
            } else if (entry.fn && typeof (entry.fn[optFn]) === "function") {
                entry.fn[optFn].apply(entry.scope || window, args);
            } else {
                console.error("No valid function associated with " + identifier
                    + " or " + identifier + "." + optFn);
            }
        } else {
            console.error("Nothing with id " + identifier + " found in storage.");
        }
    };

    instance.remove = function (identifier) {
        console.error("Remove id " + identifier + " from storage.");
        instance[identifier] = undefined;
    };
    return instance;
})();

WebLink.info = WebLink.info || {};
WebLink.info.getVersion = WebLink.info.getVersion || function () {
    return WebLinkHost.getWebLinkVersion();
};

WebLink.info.getSettings = WebLink.info.getSettings || function () {
    return JSON.parse(WebLinkHost.getWLHostAppSettings());
};

WebLink.info.setOnChangeSettingEventName = WebLink.info.setOnChangeSettingEventName || function (name) {
    WebLinkHost.setJavaScriptOnChangeSettingEventName(name);
};

WebLink.appManager.reloadApps = function () {
    WebLinkHost.reloadApps();
};

/*prevents horizontal scroll in Android 7+*/
(function () {
    var metaTagViewport = document.querySelector('meta[name=viewport]');
    if (metaTagViewport) {
        var content = metaTagViewport.getAttribute('content');
        var arr = content.split(',');
        arr.forEach(function (property, index) {
            if (property.indexOf('width') >= 0) {
                var width = arr[index].split('=');
                width[1] = window.innerWidth;
                arr[index] = width.join('=');
                content = arr.join(',');
                metaTagViewport.setAttribute('content', content);
            }
        });
    }
})();