
if (typeof (WebLink) === 'undefined') {
    WebLink = {};
}
WebLink.ServiceManager = WebLink.ServiceManager || {};
WebLink.ServiceManager.m_serviceIDMap = WebLink.ServiceManager.m_serviceIDMap || {};

WebLink.ServiceManager.registerService = function (serviceName, providerName, protocols, resourcePath, serviceObj) {
    if (typeof (WLServiceManager) !== 'undefined') {
        if (protocols instanceof Array){
            protocols = JSON.stringify(protocols);
        }
        var serviceID = WLServiceManager.registerService(serviceName, providerName, protocols, resourcePath);
        WebLink.ServiceManager.m_serviceIDMap[serviceID] = serviceObj;
        return serviceID;
    }
};
WebLink.ServiceManager.unRegisterService = function (serviceID) {
    if (typeof (WLServiceManager) !== 'undefined') {
        WLServiceManager.unregisterService(serviceID);
    }
};

WebLink.ServiceManager.onProcessRequest = function (serviceID, resourcePath, requestID, requestMethod, requestBody) {
    var serviceObj = WebLink.ServiceManager.m_serviceIDMap[serviceID];
    var request = {
        requestID: requestID,
        requestMethod: requestMethod,
        requestData: requestBody
    };
    if (typeof (serviceObj) !== 'undefined') {
        serviceObj.onProcessRequest(resourcePath, request, new WebLink.IServiceResponseNotification(serviceID));
    }
};
WebLink.ServiceManager.onRegisterNotification = function (serviceID, numberOfNotifications) {
    var serviceObj = WebLink.ServiceManager.m_serviceIDMap[serviceID];
    if (typeof (serviceObj) !== 'undefined') {
        if (numberOfNotifications > 0) {
            serviceObj.onRegisterNotification(new WebLink.IServiceResponseNotification(serviceID));
        } else {
            serviceObj.onRegisterNotification(null);
        }
    }
};
WebLink.ServiceManager.onCancelRequest = function (serviceID, requestID) {
    var serviceObj = WebLink.ServiceManager.m_serviceIDMap[serviceID];
    if (typeof (serviceObj) !== 'undefined') {
        serviceObj.onCancelRequest(requestID);
    }
};
WebLink.IServiceResponseNotification = function (serviceID) {
    this.m_serviceID = serviceID;
};
WebLink.IServiceResponseNotification.prototype.onResponseReceived = function (request, response) {
   if (typeof (WebLink.ServiceManager) !== 'undefined') {
     if (typeof response.requestBody == 'string'){
        WLServiceManager.onResponseReceivedString(this.m_serviceID, response.requestID, response.requestBody);
        return;
     } else if(response.requestBody instanceof Uint8Array){
        var bytes = [];
       for(var i=0; i <response.requestBody.length; i++){
         bytes.push(response.requestBody[i]);
       }
       response.requestBody = bytes;
     } else if (response.requestBody instanceof Array == false && typeof response.requestBody == 'object'){
       response.requestBody = JSON.stringify(response.requestBody);
       WLServiceManager.onResponseReceivedString(this.m_serviceID, response.requestID, response.requestBody);
       return;
     } else if (response.requestBody instanceof Array == false){
        throw("response payload must be array object string or Uint8Array");
     }
     WLServiceManager.onResponseReceived(this.m_serviceID, response.requestID, response.requestBody);
   }
};

WebLink.IServiceResponseNotification.prototype.onRequestFailed = function (request, errorCode, responseBody) {
    if (typeof(WLServiceManager) !== 'undefined') {
        if (typeof responseBody == 'string') {
            WLServiceManager.onRequestFailedString(this.m_serviceID, request.requestID, errorCode, responseBody);
            return;
        } else if (responseBody instanceof Uint8Array) {
            var bytes = [];
            for (var i = 0; i < responseBody.length; i++) {
                bytes.push(responseBody[i]);
            }
            responseBody = bytes;
        } else if (responseBody instanceof Array == false && typeof responseBody == 'object') {
            responseBody = JSON.stringify(responseBody);
            WLServiceManager.onRequestFailedString(this.m_serviceID, request.requestID, errorCode, responseBody);
            return;
        } else if (responseBody instanceof Array == false) {
            throw ("Response payload must be array, object, string or Uint8Array");
        }
        WLServiceManager.onRequestFailed(this.m_serviceID, request.requestID, errorCode, responseBody);
    }
};

WebLink.IServiceResponseNotification.prototype.onNotify = function (resourcePath, data) {
    if (typeof (WLServiceManager) !== 'undefined') {
        if (typeof data == 'string'){
           WLServiceManager.onNotifyString(this.m_serviceID, resourcePath, data);
           return;
        } else if(data instanceof Uint8Array){
           var bytes = [];
           for(var i=0; i <data.length; i++){
             bytes.push(data[i]);
           }
           data = bytes;
        } else if (data instanceof Array == false && typeof data == 'object'){
          data = JSON.stringify(data);
          WLServiceManager.onNotifyString(this.m_serviceID, resourcePath, data);
          return;
        } else if (data instanceof Array == false){
           throw("response payload must be array, object, string or Uint8Array");
        }
        WLServiceManager.onNotify(this.m_serviceID, resourcePath, data);
    }
};

WebLink.EServiceErrorCode = {
    NotAvailable: 0,
    AccessDenied: 1,
    Busy: 2,
    UnsupportedRequest: 3,
    InvalidArgument: 4,
    CancelledByCaller: 5
};
WebLink.Response = function (requestID, responseBody) {
    this.requestID = requestID;
    this.requestBody = responseBody;
};
WebLink.Request = function (requestID, requestBody, requestMethod) {
    this.requestID = requestID;
    this.requestBody = requestBody;
    this.requestMethod = requestMethod;
    this.allowExecuteInForeground = true;
};

WebLink.wlappservices = WebLink.wlappservices || {};
WebLink.wlappservices.discoveryNotifications = WebLink.wlappservices.discoveryNotifications || {};
WebLink.wlappservices.requests = WebLink.wlappservices.requests || {};
WebLink.wlappservices.serviceNotifications = WebLink.wlappservices.serviceNotifications || {};

WebLink.wlappservices.findServiceByName = function(name, serviceDiscoveryNotification) {
  var searchID = WLServiceManager.findServiceByName(name);
  WebLink.wlappservices.discoveryNotifications[searchID] = serviceDiscoveryNotification;
};

WebLink.wlappservices._proxy = function (descriptorID, name) {
  this.name = name;
  this.descriptorID = descriptorID;
  
  this.sendRequest = function(resourcePath, requestMethod, requestBody, serviceResponseNotification) {
  	var requestID = WLServiceManager.sendRequest(this.descriptorID, resourcePath, requestMethod, requestBody);
  	WebLink.wlappservices.requests[requestID] = serviceResponseNotification;
  	return requestID;
  };
  
  this.registerForNotification = function(resourcePath, notificationHandler) {
  	var notificationID = WLServiceManager.registerNotification(this.descriptorID, resourcePath);
    if(notificationID > 0) {
  	  WebLink.wlappservices.serviceNotifications[notificationID] = notificationHandler;
    }
  	return notificationID;
  };
  
  this.unregisterFromNotification = function(resourcePath, notificationID) {
    if(notificationID > 0) {
  	  WebLink.wlappservices.serviceNotifications[notificationID] = undefined;
   	  WLServiceManager.unregisterNotification(this.descriptorID, resourcePath, notificationID);
    }
  };
};

WebLink.wlappservices._onServiceFound = function(searchID, descriptorID, index, name) {
  var notification = WebLink.wlappservices.discoveryNotifications[searchID];
  if(typeof(notification) === 'object' && notification && typeof(notification.onServiceFound) === 'function') {
    var proxy = new WebLink.wlappservices._proxy(descriptorID, name);
    notification.onServiceFound(proxy, index);
  }
};

WebLink.wlappservices._onServiceDiscoveryCompleted = function(searchID, count) {
  var notification = WebLink.wlappservices.discoveryNotifications[searchID];
  if(typeof(notification) === 'object' && notification && typeof(notification.onServiceDiscoveryCompleted) === 'function') {
    notification.onServiceDiscoveryCompleted(count);
  }
};

WebLink.wlappservices._onServiceDiscoveryFailed = function(searchID, errorCode) {
  var notification = WebLink.wlappservices.discoveryNotifications[searchID];
  if(typeof(notification) === 'object' && notification && typeof(notification.onServiceDiscoveryFailed) === 'function') {
    notification.onServiceDiscoveryFailed(errorCode);
  }
};

WebLink.wlappservices._onServiceRequestFailed = function(requestID, errorCode, responseBody) {
  var notification = WebLink.wlappservices.requests[requestID];
  if(typeof(notification) === 'object' && notification && typeof(notification.onRequestFailed) === 'function') {
    notification.onRequestFailed(requestID, errorCode, responseBody);
  }
};

WebLink.wlappservices._onServiceResponse = function(requestID, responseBody) {
  var notification = WebLink.wlappservices.requests[requestID];
  if(typeof(notification) === 'object' && notification && typeof(notification.onResponseReceived) === 'function') {
    notification.onResponseReceived(requestID, responseBody);
  }
};

WebLink.wlappservices._onServiceNotification = function(notificationID, resourcePath, notificationData) {
  var notification = WebLink.wlappservices.serviceNotifications[notificationID];
  if(typeof(notification) === 'object' && notification && typeof(notification.onNotification) === 'function') {
    notification.onNotification(resourcePath, notificationData);
  }
};
